@@ -23,8 +23,10 @@ class Dot(LayoutEngine):
         self.fm = QFontMetrics(self.font)
         self.margins=20
         self.show_subgraphs = show_subgraphs
+        self.graph.depth_x_pos = [0]
 
-    def process(self, n, graph, index=0, nb_brothers=0):
+
+    def process(self, n, graph, index=0, nb_brothers=0, depth=0, stage=0):
         n.processed+=1
         width = 0
         height = 0
@@ -33,10 +35,15 @@ class Dot(LayoutEngine):
             n.pos[1]=n.kwargs["pos"][1]
             n.size[0]=n.kwargs["size"][0]
             n.size[1]=n.kwargs["size"][1]
-            graph.current_x += n.size[0] + self.default_min_nodes_dist
+
+            if len(graph.depth_x_pos)>depth:
+                graph.depth_x_pos[depth] += n.size[0] + self.default_min_nodes_dist
+            else:
+                graph.depth_x_pos.append(n.size[0] + self.default_min_nodes_dist)
+
             for i,oe in enumerate(n.out_edges):
                 if (oe.dest.processed<20):
-                    self.process(oe.dest, graph, i,len(n.out_edges))
+                    self.process(oe.dest, graph, i,len(n.out_edges), depth=depth+1)
 
             return
 
@@ -51,54 +58,42 @@ class Dot(LayoutEngine):
             height=self.default_node_height
 
         if(type(n)==Graph and self.show_subgraphs):
-            if len(n.nodes)>0:
-                minx=10000
-                miny=10000
-                maxx=0
-                maxy=0
-                self.build_graph(n)
-                for i, node in enumerate(n.nodes):
-
-                    if(node.pos[0]<minx):
-                        minx = node.pos[0]-node.size[0]/2
-                    if(node.pos[1]<miny):
-                        miny = node.pos[1]-node.size[1]/2
-
-                    if(node.pos[0]>maxx):
-                        maxx = node.pos[0]+node.size[0]/2
-                    if(node.pos[1]>maxy):
-                        maxy = node.pos[1]+node.size[1]/2
-
-                w=maxx-minx+2*self.margins
-                h=maxy-miny+2*self.margins
-                width = w if w>width else width
-                height = h if h>height else height
-            else:
-                graph.size=[width, height]
+            n.depth_x_pos=[0]
+            for nn in n.nodes:
+                self.process(nn, n, 0, len(n.nodes), depth=depth)
+            _,_,w_,h_=n.getRect()
+            #w_+=2*self.default_min_nodes_dist
+            width = w_ if w_>width else width
+            height = h_ if h_>height else height
+            width += 2*self.margins
+            height += 2*self.margins
+            
         
 
         n.size[0]=width
         n.size[1]=height
 
         if len(n.in_edges)==0:
-            n.pos[0]=graph.current_x
-            n.pos[1]=self.default_node_height/2
-
-            graph.current_x += width + self.default_min_nodes_dist
+            n.pos[0]=graph.depth_x_pos[depth]+self.margins+width/2
+            n.pos[1]=self.default_node_height/2+self.margins+height/2
+            if len(graph.depth_x_pos)>depth:
+                graph.depth_x_pos[depth] += width/2 + self.default_min_nodes_dist
+            else:
+                graph.depth_x_pos.append(width/2 + self.default_min_nodes_dist)
         else:
-            x=(width + self.default_min_nodes_dist)*(-(nb_brothers-1)/2+index)
+            x=(width/2 + self.default_min_nodes_dist)*(-(nb_brothers-1)/2+index)
             y=0
             for i,oe in enumerate(n.out_edges):
                 if (oe.dest.processed<20):
-                    self.process(oe.dest, graph, i,len(n.out_edges))
+                    self.process(oe.dest, graph, i,len(n.out_edges),depth=depth+1)
 
             for edg in n.in_edges:
                 if (edg.source.processed==0):
                     self.process(edg.source, graph)
                 if(n.parent_graph == edg.source.parent_graph):
                     x += edg.source.pos[0]
-                    if(y<edg.source.pos[1]+width/2+self.default_min_nodes_dist):
-                        y = edg.source.pos[1]+width/2+self.default_min_nodes_dist
+                    if(y<edg.source.pos[1]+height/2+self.default_min_nodes_dist):
+                        y = edg.source.pos[1]+height/2+self.default_min_nodes_dist
             x/=len(n.in_edges)
 
             n.pos[0]=x
@@ -108,6 +103,12 @@ class Dot(LayoutEngine):
             if oe.dest.processed<20:
                 self.process(oe.dest, graph, i,len(n.out_edges))
 
+        if(type(n)==Graph and self.show_subgraphs):
+            n.depth_x_pos=[0]
+            for nn in n.nodes:
+                self.process(nn, n, 0, len(n.nodes), depth=depth)
+            
+
     def build_graph(self, graph):
         for n in graph.nodes:
             n.processed=0
@@ -115,17 +116,18 @@ class Dot(LayoutEngine):
         for i,n in enumerate(graph.nodes):
             if(n.processed<3):
                 self.process(n, graph)
-
+        """
         for n in graph.nodes: 
             if(n.pos[0]<n.size[0]/2):
                 for node in graph.nodes:
                     node.pos[0]+=(n.size[0]/2)-n.pos[0]
-        
-
+        """
+        _,_,graph.size[0], graph.size[1] = graph.getRect()
+        graph.pos[0] = graph.size[0] /2
+        graph.pos[1] = graph.size[1] /2
 
 
     def build(self):
-        self.graph.current_x=self.default_node_width/2
-        self.graph.current_y =self.default_node_height/2
+        self.graph.depth_x_pos = [0]
 
         self.build_graph(self.graph)
