@@ -8,7 +8,7 @@ Main Class to QGraphViz tool
 """
 from PyQt5.QtWidgets import QApplication, QWidget, QScrollArea, QSizePolicy
 from PyQt5.QtGui import QPainter, QPen, QBrush, QColor, QPainterPath, QImage, QLinearGradient
-from PyQt5.QtCore import Qt, QRect
+from PyQt5.QtCore import Qt, QRect, QRectF
 import os
 import sys
 import enum
@@ -248,9 +248,11 @@ class QGraphViz_Core(QWidget):
             self.hovered_Node = None
 
         self.parser.save(filename, self.engine.graph)
+        self.current_path=filename
 
     def saveAsJson(self, filename):
         self.parser.toJSON(filename, self.engine.graph)
+        self.current_path=filename
 
 
     # ================== Helper methods ==================
@@ -559,11 +561,6 @@ class QGraphViz_Core(QWidget):
         pen=QPen()
         brush.setColor(Qt.white)
 
-        if(self.show_subgraphs):
-            for node in graph.nodes:
-                if type(node)==Graph:
-                    subgraph = node
-                    self.paintSubgraph(subgraph, painter, pen, brush)
 
         for i,edge in enumerate(graph.edges):
             if("color" in edge.kwargs.keys()):
@@ -608,6 +605,13 @@ class QGraphViz_Core(QWidget):
             gdpos[0],
             gdpos[1])
             """
+
+        if(self.show_subgraphs):
+            for node in graph.nodes:
+                if type(node)==Graph:
+                    subgraph = node
+                    self.paintSubgraph(subgraph, painter, pen, brush)
+                    
          # TODO : add more painting parameters
         for node in graph.nodes:
             if type(node)!=Graph:
@@ -646,23 +650,89 @@ class QGraphViz_Core(QWidget):
                                     gpos[1]-node.size[1]/2,
                                     node.size[0], node.size[1])
 
-                    if(node.kwargs["shape"]=="circle"):
+                    elif(node.kwargs["shape"]=="circle"):
                         painter.drawEllipse(
                                     gpos[0]-node.size[0]/2,
                                     gpos[1]-node.size[1]/2,
                                     node.size[0], node.size[1])
-                    
-                    # Image as a node, this implementation checks to see if a 
-                    # file path was provided in the shape parameter
-                    if(os.path.isfile(node.kwargs["shape"])): 
-                        img_path = node.kwargs["shape"]
-                        painter.drawImage(
-                            QRect(
-                                gpos[0]-node.size[0]/2,
-                                gpos[1]-node.size[1]/2,
-                                node.size[0],
-                                node.size[1]), 
-                            QImage(img_path))
+                    elif(node.kwargs["shape"]=="triangle"):
+                        rect = QRect(gpos[0]-node.size[0]/2, gpos[1]-2*node.size[1]/3, node.size[0], node.size[1])
+
+                        path = QPainterPath()
+                        path.moveTo(rect.left() + (rect.width() / 2), rect.top())
+                        path.lineTo(rect.bottomLeft())
+                        path.lineTo(rect.bottomRight())
+                        path.lineTo(rect.left() + (rect.width() / 2), rect.top())
+
+                        painter.fillPath(path, brush)
+                        painter.drawPath(path)
+                    elif(node.kwargs["shape"]=="polygon"):
+                        rect = QRect(gpos[0]-node.size[0]/2, gpos[1]-node.size[1]/2, node.size[0], node.size[1])
+
+                        path = QPainterPath()
+                        path.moveTo(rect.left() + (rect.width() / 4), rect.top())
+                        path.lineTo(rect.left() + 3*rect.width()/4, rect.top())
+                        path.lineTo(rect.left() + rect.width(), rect.top() + rect.height()/2)
+                        path.lineTo(rect.left() + 3*rect.width()/4, rect.top() + rect.height())
+                        path.lineTo(rect.left() + rect.width()/4, rect.top() + rect.height())
+                        path.lineTo(rect.left(), rect.top() + rect.height()/2)
+                        path.lineTo(rect.left() + (rect.width() / 4), rect.top())
+ 
+                        painter.fillPath(path, brush)
+                        painter.drawPath(path)
+                    elif(node.kwargs["shape"]=="diamond"):
+                        rect = QRect(gpos[0]-node.size[0]/2, gpos[1]-node.size[1]/2, node.size[0], node.size[1])
+
+                        path = QPainterPath()
+                        path.moveTo(rect.left() + (rect.width() / 2), rect.top())
+                        path.lineTo(rect.left() + rect.width(), rect.top() + rect.height()/2)
+                        path.lineTo(rect.left() + rect.width()/2, rect.top() + rect.height())
+                        path.lineTo(rect.left(), rect.top() + rect.height()/2)
+                        path.lineTo(rect.left() + (rect.width() / 2), rect.top())
+ 
+                        painter.fillPath(path, brush)
+                        painter.drawPath(path)
+
+                    else: # assuming this is an image
+                        # this parameter can be either direct image path
+                        # or a relative path (relative to the file path)
+                        # It can contain the format path,width,height
+                        # or simple path in which case the image file size will be used
+                        image = None
+                        width = 0
+                        height = 0
+                        if("," in node.kwargs["shape"]): # if there is a , in the shape, the first part is the path, then width, then height
+                            img_params = node.kwargs["shape"].split(",")
+                            if len(img_params)==3:# img:width:height
+                                img_path = img_params[0]
+                                width =  int(img_params[1])
+                                height =  int(img_params[2])
+                                img_path2 = os.path.join(os.path.dirname(self.current_path),img_path)
+                                if(os.path.isfile(img_path)):
+                                    image = QImage(img_path)
+                                elif(os.path.isfile(img_path2)):
+                                    image = QImage(img_path)
+                        else:
+                            img_path = node.kwargs["shape"]
+                            img_path2 = os.path.join(os.path.dirname(self.current_path),img_path)
+                            if(os.path.isfile(img_path)):
+                                image = QImage(img_path)
+                                width =  image.size().width()
+                                height =  image.size().height()
+                            elif(os.path.isfile(img_path2)):
+                                image = QImage(img_path)
+                                width =  image.size().width()
+                                height =  image.size().height()
+                        if image is not None:
+                            node.size[0] = width
+                            node.size[1] = height
+                            painter.drawImage(
+                                QRect(
+                                    gpos[0]-node.size[0]/2,
+                                    gpos[1]-node.size[1]/2,
+                                    node.size[0],
+                                    node.size[1]), 
+                                image)
                 else:
                     painter.drawEllipse(
                                 gpos[0]-node.size[0]/2,
